==> oobium_datastore-1.0.0-dev.6.tar.gz: lib/src/models/collection.dart <==
import 'package:collection/src/iterable_extensions.dart';
import 'package:objectid/objectid.dart';
import 'package:oobium_datastore/src/models/model.dart';
import 'package:oobium_datastore/src/state.dart';

class ModelCollection<T extends Model<T, I>, I, R extends ModelRef<T, I, R>> implements Iterable<R> {

  final WatchableStateFeature _feature;
  final R Function(T s) _ref;
  final bool Function(T s)? _test;
  final _tests = <bool Function(R element)>[];
  ModelCollection(this._feature, this._ref, [this._test]);

  Iterable<R> get _all => _applyTo(_feature.stateValues);

  @override
  bool any(bool Function(R element) test) => _all.any(test);
  bool none(bool Function(R element) test) => !any(test);

  @override
  Iterable<A> cast<A>() => _all.cast<A>();

  @override
  bool contains(Object? obj) {
    if(obj is ObjectId || obj is int || obj is String) obj = _Key<T>(obj);
    return _all.contains(obj);
  }

  @override
  R elementAt(int index) => _all.elementAt(index);

  @override
  bool every(bool Function(R element) test) => _all.every(test);

  @override
  Iterable<A> expand<A>(Iterable<A> Function(R element) toElements) => _all.expand(toElements);

  @override
  R get first => _all.first;
  R? get firstOrNull => _all.firstOrNull;

  @override
  R firstWhere(bool Function(R element) test, {R Function()? orElse}) => _all.firstWhere(test, orElse: orElse);
  R? firstWhereOrNull(bool Function(R element) test) => _all.firstWhereOrNull(test);

  @override
  A fold<A>(A initialValue, A Function(A previousValue, R element) combine) => _all.fold(initialValue, combine);

  @override
  Iterable<R> followedBy(Iterable<R> other) => _all.followedBy(other);

  @override
  void forEach(void Function(R element) action) => _all.forEach(action);

  @override
  bool get isEmpty => _all.isEmpty;

  @override
  bool get isNotEmpty => _all.isNotEmpty;

  @override
  Iterator<R> get iterator => _all.iterator;

  @override
  String join([String separator = ""]) => _all.join(separator);

  @override
  R get last => _all.last;
  R? get lastOrNull => _all.lastOrNull;

  @override
  R lastWhere(bool Function(R element) test, {R Function()? orElse}) => _all.lastWhere(test, orElse: orElse);
  R? lastWhereOrNull(bool Function(R element) test) => _all.lastWhereOrNull(test);

  @override
  int get length => _all.length;

  @override
  Iterable<A> map<A>(A Function(R e) toElement) => _all.map(toElement);

  @override
  R reduce(R Function(R value, R element) combine) => _all.reduce(combine);

  @override
  R get single => _all.single;
  R? get singleOrNull => _all.singleOrNull;

  @override
  R singleWhere(bool Function(R element) test, {R Function()? orElse}) => _all.singleWhere(test, orElse: orElse);
  R? singleWhereOrNull(bool Function(R element) test) => _all.singleWhere(test);

  @override
  Iterable<R> skip(int count) => _all.skip(count);

  @override
  Iterable<R> skipWhile(bool Function(R value) test) => _all.skipWhile(test);

  @override
  Iterable<R> take(int count) => _all.take(count);

  @override
  Iterable<R> takeWhile(bool Function(R value) test) => _all.takeWhile(test);

  @override
  List<R> toList({bool growable = true}) => _all.toList(growable: growable);

  @override
  Set<R> toSet() => _all.toSet();

  @override
  ModelCollection<T, I, R> where(bool Function(R element) test) =>
      ModelCollection(_feature, _ref, _test).._tests.add(test);

  @override
  Iterable<A> whereType<A>() => _all.whereType<A>();

  I operator <<(T model) {
    _feature.put(model);
    return model.id;
  }

  R load(T model) {
    _feature.put(model);
    return _ref(model);
  }

  bool remove() {
    var removed = false;
    for(final r in _all.toList()) {
      if(r.remove() != null) {
        removed = true;
      }
    }
    return removed;
  }

  List<R> set(Iterable<T> models, {bool Function(R m)? where}) {
    if (where != null) {
      _feature.set<T>(models, where: (m) => where(_ref(m)));
    } else {
      _feature.set<T>(models);
    }
    return models.map((m) => _ref(m)).toList();
  }

  Stream<Iterable<R>> stream({bool immediate = true}) async* {
    if (immediate) {
      yield this;
    }
    yield* watch().map((_) => this);
  }

  Stream<Iterable<R>> watch() => _feature.watch()
      .map((events) => _applyTo(events.values));

  Iterable<R> _applyTo(Iterable<StateValue> iterable) {
    final tQuery = (_test == null)
        ? iterable.whereType<T>()
        : iterable.whereType<T>().where(_test!);
    var rQuery = tQuery.map(_ref);
    for(final test in _tests) {
      rQuery = rQuery.where(test);
    }
    return rQuery;
  }
}

class _Key<T> extends StateKey<T> {
  final dynamic id;
  _Key(this.id);
}

==> oobium_datastore-1.1.1.tar.gz: lib/src/models/collection.dart <==
import 'package:collection/src/iterable_extensions.dart';
import 'package:objectid/objectid.dart';
import 'package:oobium_datastore/src/models/model.dart';
import 'package:oobium_datastore/src/state.dart';

class ModelCollection<C extends ModelCollection<C, T, I, R>, T extends Model<T, I>, I, R extends ModelRef<T, I, R>> implements Iterable<R> {

  final WatchableStateFeature _feature;
  final C Function() _new;
  final R Function(T s) _ref;
  final bool Function(T s)? _test;
  final _tests = <bool Function(R element)>[];
  ModelCollection(this._feature, this._new, this._ref, [this._test]);

  Iterable<R> get _all => _applyTo(_feature.stateValues);

  @override
  bool any(bool Function(R element) test) => _all.any(test);
  bool none(bool Function(R element) test) => !any(test);

  @override
  Iterable<A> cast<A>() => _all.cast<A>();

  @override
  bool contains(Object? obj) {
    if(obj is ObjectId || obj is int || obj is String) obj = _Key<T>(obj);
    return _all.contains(obj);
  }

  @override
  R elementAt(int index) => _all.elementAt(index);

  @override
  bool every(bool Function(R element) test) => _all.every(test);

  @override
  Iterable<A> expand<A>(Iterable<A> Function(R element) toElements) => _all.expand(toElements);

  @override
  R get first => _all.first;
  R? get firstOrNull => _all.firstOrNull;

  @override
  R firstWhere(bool Function(R element) test, {R Function()? orElse}) => _all.firstWhere(test, orElse: orElse);
  R? firstWhereOrNull(bool Function(R element) test) => _all.firstWhereOrNull(test);

  @override
  A fold<A>(A initialValue, A Function(A previousValue, R element) combine) => _all.fold(initialValue, combine);

  @override
  Iterable<R> followedBy(Iterable<R> other) => _all.followedBy(other);

  @override
  void forEach(void Function(R element) action) => _all.forEach(action);

  @override
  bool get isEmpty => _all.isEmpty;

  @override
  bool get isNotEmpty => _all.isNotEmpty;

  @override
  Iterator<R> get iterator => _all.iterator;

  @override
  String join([String separator = ""]) => _all.join(separator);

  @override
  R get last => _all.last;
  R? get lastOrNull => _all.lastOrNull;

  @override
  R lastWhere(bool Function(R element) test, {R Function()? orElse}) => _all.lastWhere(test, orElse: orElse);
  R? lastWhereOrNull(bool Function(R element) test) => _all.lastWhereOrNull(test);

  @override
  int get length => _all.length;

  @override
  Iterable<A> map<A>(A Function(R e) toElement) => _all.map(toElement);

  @override
  R reduce(R Function(R value, R element) combine) => _all.reduce(combine);

  @override
  R get single => _all.single;
  R? get singleOrNull => _all.singleOrNull;

  @override
  R singleWhere(bool Function(R element) test, {R Function()? orElse}) => _all.singleWhere(test, orElse: orElse);
  R? singleWhereOrNull(bool Function(R element) test) => _all.singleWhere(test);

  @override
  Iterable<R> skip(int count) => _all.skip(count);

  @override
  Iterable<R> skipWhile(bool Function(R value) test) => _all.skipWhile(test);

  @override
  Iterable<R> take(int count) => _all.take(count);

  @override
  Iterable<R> takeWhile(bool Function(R value) test) => _all.takeWhile(test);

  @override
  List<R> toList({bool growable = true}) => _all.toList(growable: growable);

  @override
  Set<R> toSet() => _all.toSet();

  @override
  C where(bool Function(R element) test) => _new().._tests.add(test);

  @override
  Iterable<A> whereType<A>() => _all.whereType<A>();

  I operator <<(T model) {
    _feature.put(model);
    return model.id;
  }

  R load(T model) {
    _feature.put(model);
    return _ref(model);
  }

  List<R> loadAll(Iterable<T> state) => state.map((s) => load(s)).toList();

  bool remove() {
    var removed = false;
    for(final r in _all.toList()) {
      if(r.remove() != null) {
        removed = true;
      }
    }
    return removed;
  }

  List<R> set(Iterable<T> models, {bool Function(R m)? where}) {
    if (where != null) {
      _feature.set<T>(models, where: (m) => where(_ref(m)));
    } else {
      _feature.set<T>(models);
    }
    return models.map((m) => _ref(m)).toList();
  }

  Stream<C> stream({bool immediate = true}) async* {
    if (immediate) {
      yield this as C;
    }
    yield* watch().map((_) => this as C);
  }

  Stream<Iterable<R>> watch() => _feature.watch()
      .map((events) => _applyTo(events.values));

  Iterable<R> _applyTo(Iterable<StateValue> iterable) {
    final tQuery = (_test == null)
        ? iterable.whereType<T>()
        : iterable.whereType<T>().where(_test!);
    var rQuery = tQuery.map(_ref);
    for(final test in _tests) {
      rQuery = rQuery.where(test);
    }
    return rQuery;
  }
}

class _Key<T> extends StateKey<T> {
  final dynamic id;
  _Key(this.id);
}
